import {
    Component, ElementRef, Output,
    EventEmitter, NgZone, OnDestroy,
    trigger, transition, style, animate
} from '@angular/core';
import { AngularDatepickerOptions } from '../datepicker-options';

import moment from 'moment';

export interface CalendarDate {
    day: number;
    month: number;
    year: number;
    differentMonth: boolean;
    enabled: boolean;
    today: boolean;
    weekend: boolean;
    selected: boolean;
}

export interface Week {
    rowId?: number;
    weekNumber?: number;
    days?: Array<CalendarDate>
}

@Component({
    selector: 'jb-datepicker-container',
    template: `
      <div class="calendar-wrapper" [class.modal]="mqMatches" *ngIf="opened" [@contentAnimation]="opened">
        <div class="overlay" (click)="close()"></div>
        <div class="calendar-container">
          <div class="calendar-top">
            <div class="top-chevron" (click)="prevMonth()">
              <svg fill="#FFFFFF" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg">
                <path d="M15.41 7.41L14 6l-6 6 6 6 1.41-1.41L10.83 12z" />
                <path d="M0 0h24v24H0z" fill="none" />
              </svg>
            </div>
            <div class="top-month-year">
              <span>{{ date.format('MMMM YYYY') }}</span>
            </div>
            <div class="top-chevron" (click)="nextMonth()">
              <svg fill="#FFFFFF" height="24" viewBox="0 0 24 24" width="24" xmlns="http://www.w3.org/2000/svg">
                <path d="M10 6L8.59 7.41 13.17 12l-4.58 4.59L10 18l6-6z" />
                <path d="M0 0h24v24H0z" fill="none" />
              </svg>
            </div>
          </div>
          <table>
            <thead>
              <tr class="day-names">
                <th *ngFor="let name of weekDayNames">{{name}}</th>
              </tr>
            </thead>
            <tbody class="days" [class.weekend-highlight]="options.weekendHighlight" [class.today-highlight]="options.todayHighlight">
              <tr *ngFor="let week of weeks">
                <td *ngFor="let d of week.days" class="day" [class.weekend]="d.weekend">
                  <span (click)="selectDate($event, d)" [class.today]="d.today" [class.different-month]="!d.differentMonth" [class.is-active]="d.selected">{{d.day}}</span>
                </td>
              </tr>
              <tr>
                <td colspan="7" class="today-button-wrapper">
                  <button type="button" (click)="setToday()">{{options.todayString}}</button>
                </td>
              </tr>
            </tbody>
          </table>
        </div>
      </div>
    `,
    styles: [`
      :host .calendar-wrapper{position:absolute;z-index:9999;min-width:200px}:host .calendar-wrapper .overlay{display:none}:host .calendar-wrapper .calendar-container{min-width:90%;z-index:999;background-color:#fff;box-shadow:0 1px 3px rgba(0,0,0,0.12),0 1px 2px rgba(0,0,0,0.24)}:host .calendar-wrapper .calendar-container .calendar-top{width:100%;min-height:40px;color:#fff;background-color:#613bbb;line-height:40px;display:flex;align-items:center;justify-content:space-between}:host .calendar-wrapper .calendar-container .calendar-top .top-chevron{cursor:pointer;padding:0.5em;outline:none}:host .calendar-wrapper .calendar-container .calendar-top .top-chevron svg{vertical-align:middle}:host .calendar-wrapper .calendar-container .calendar-top .top-month-year{padding:0.5em;outline:none}:host .calendar-wrapper .calendar-container .calendar-top .top-month-year span{text-align:center;font-size:1.2em;vertical-align:middle}:host .calendar-wrapper .calendar-container table{width:100%;border-collapse:collapse;text-align:center}:host .calendar-wrapper .calendar-container .day-names{background:#7C4DFF;padding-left:5px;padding-right:5px}:host .calendar-wrapper .calendar-container .day-names th{width:calc(100% / 7);font-size:1em;text-align:center;padding-top:5px;padding-bottom:5px;color:#fff}:host .calendar-wrapper .calendar-container .days{margin-left:5px;margin-right:5px}:host .calendar-wrapper .calendar-container .days.weekend-highlight .day.weekend{background-color:rgba(0,0,0,0.1)}:host .calendar-wrapper .calendar-container .days.today-highlight .day span.today{font-weight:bold;color:#fff;background:rgba(97,59,187,0.4)}:host .calendar-wrapper .calendar-container .days .day{cursor:pointer;text-align:center}:host .calendar-wrapper .calendar-container .days .day span{font-size:1em;display:inline-block;text-align:center;height:2em;width:2em;line-height:2em;border-radius:50%;margin:2px}:host .calendar-wrapper .calendar-container .days .day span.different-month{color:#ccc}:host .calendar-wrapper .calendar-container .days .day span:hover,:host .calendar-wrapper .calendar-container .days .day span.is-active,:host .calendar-wrapper .calendar-container .days .day span.is-active.today{background:#613bbb;color:#fff}:host .calendar-wrapper .calendar-container .today-button-wrapper{padding-bottom:5px;padding-top:5px}:host .calendar-wrapper .calendar-container .today-button-wrapper button{display:flex;margin:auto;align-items:center;align-self:center;justify-content:center;margin-bottom:0;font-weight:400;text-align:center;vertical-align:middle;cursor:pointer;background-image:none;border:1px solid transparent;white-space:nowrap;padding:5px 10px;font-size:1em;line-height:1.4;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;outline:none}:host .calendar-wrapper.modal{display:flex;flex-flow:column;position:fixed;top:0;left:0;width:100%;height:100%;justify-content:center;align-items:center}:host .calendar-wrapper.modal .overlay{position:absolute;top:0;left:0;width:100%;height:100%;display:flex;z-index:900;justify-content:center;align-items:center;background-color:rgba(0,0,0,0.7)}
    `],
    animations: [
        trigger('contentAnimation', [
            transition('void => *', [
                style({ opacity: '0' }),
                animate('200ms ease-in')
            ]),
            transition('* => void', [
                animate('300ms ease-out', style({ opacity: '0' }))
            ])
        ])
    ]
})
export class DatePickerContainerComponent implements OnDestroy {
    @Output() onDateSelected = new EventEmitter();

    public opened: boolean = false;
    public date: moment.Moment = moment();
    public inputDate: moment.Moment;
    public weeks: Array<Week>;
    public weekDayNames: Array<string>;
    public days: Array<any> = [];
    public mqMatches: boolean;
    public options: AngularDatepickerOptions = {};

    public weekendHighlight: boolean;
    public todayHighlight: boolean;
    public todayString: boolean;

    private firstWeekdaySunday: boolean;

    private mq: MediaQueryList;

    private watchMediaQuery = (mql: MediaQueryList) => {
        this.zone.run(() => {
            this.mqMatches = this.mq.matches ? true : false;
        });
    }

    constructor(private elRef: ElementRef,
        private zone: NgZone) {
    }

    mergeOptions(options: AngularDatepickerOptions) {
        Object.assign(this.options, options);
    }

    generateCalendar(selectedMonthFlag?: boolean) {
        this.weekDayNames = moment.weekdaysShort();
        if (!this.firstWeekdaySunday) {
            this.weekDayNames.splice(6, 0, this.weekDayNames.splice(0, 1)[0]);
        }
        this.date = selectedMonthFlag && this.inputDate ? this.inputDate.clone() : moment(this.date);
        let month = this.date.month();
        let firstDay = this.date.clone().startOf('month');
        let offset;
        if (this.firstWeekdaySunday) {
            offset = firstDay.day() === 0 ? 7 : firstDay.day();
        } else {
            offset = firstDay.day() === 0 ? 6 : firstDay.day() - 1;
        }
        let daysInMonth = this.date.daysInMonth();
        let rows = Math.ceil((offset + daysInMonth) / 7);

        this.weeks = [];

        let currentDate = firstDay.subtract(offset, 'day');
        let selectedDate = this.inputDate;

        for (let i = 1; i <= rows; i += 1) {
            let days: Array<any> = [];
            for (let i = 1; i <= 7; i += 1) {
                let today = (moment().isSame(currentDate, 'day') && moment().isSame(currentDate, 'month')) ? true : false;
                let weekend = (moment(currentDate).day() == 6) || (moment(currentDate).day() == 0);
                let selected = (selectedDate && selectedDate.isSame(currentDate, 'day')) ? true : false;
                days.push({
                    day: currentDate.date(),
                    month: currentDate.month() + 1,
                    year: currentDate.year(),
                    enabled: true,
                    differentMonth: month === currentDate.month(),
                    today: today,
                    weekend: weekend,
                    selected: selected
                });
                currentDate.add(1, 'day');
            }
            this.weeks.push({ rowId: i, days: days });
        }
    }

    setToday() {
        let today = moment();
        this.inputDate = today;
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    }

    selectDate(e: MouseEvent, d: CalendarDate) {
        e.preventDefault();

        this.inputDate = moment(`${d.day}.${d.month}.${d.year}`, 'DD.MM.YYYY');
        this.generateCalendar(true);
        this.onDateSelected.emit(this.inputDate);
    }

    prevMonth() {
        this.date = this.date.subtract(1, 'month');
        this.generateCalendar();
    }

    nextMonth() {
        this.date = this.date.add(1, 'month');
        this.generateCalendar();
    }

    open() {
        this.opened = true;
        this.generateCalendar(true);
        if (this.options.modalMediaQuery) {
            this.mq = window.matchMedia(`(${this.options.modalMediaQuery})`);
            this.mqMatches = this.mq.matches ? true : false;
            this.mq.addListener(this.watchMediaQuery);
        }
    }

    close() {
        this.opened = false;
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    }

    ngOnDestroy() {
        if (this.mq) {
            this.mq.removeListener(this.watchMediaQuery);
        }
    }
}
